@@ -5,6 +5,7 @@
 **  Revised:     February 23, 2015
 **  Revised:     May 5, 2015
 **  Revised:     June 15, 2015
+**  Revised:     November 5, 2015
 **  Description:
 **    For each grid cell, this function determines the set of shapefile records
 **    contained in the cell and returns the shapefile record IDs and the clipped
@@ -107,7 +108,7 @@ SEXP insideAreaGridCell(SEXP fileNamePrefix, SEXP dsgnmdIDVec, SEXP cellIDsVec,
 
     /* create the full .shp file name */
     fileNameLen = strlen(CHAR(STRING_ELT(fileNamePrefix, 0))) + strlen(shpExt);
-    if ((shpFileName = (char * restrict)malloc(fileNameLen + 1)) == NULL ) {
+    if ((shpFileName = (char * restrict) malloc(fileNameLen + 1)) == NULL ) {
       Rprintf( "Error: Allocating memory in C function insideAreaGridCell\n" );
       PROTECT( results = allocVector( VECSXP, 1 ) );
       UNPROTECT( 1 );
@@ -121,6 +122,7 @@ SEXP insideAreaGridCell(SEXP fileNamePrefix, SEXP dsgnmdIDVec, SEXP cellIDsVec,
   /* create the new temporary .shp file */
   if((newShp = fopen(TEMP_SHP_FILE, "wb")) == NULL) {
     Rprintf("Error: Creating temporary .shp file %s in C function insideAreaGridCell.\n", TEMP_SHP_FILE);
+    free( shpFileName );
     PROTECT(results = allocVector(VECSXP, 1));
     UNPROTECT(1);
     return results;
@@ -131,6 +133,7 @@ SEXP insideAreaGridCell(SEXP fileNamePrefix, SEXP dsgnmdIDVec, SEXP cellIDsVec,
     Rprintf("Error: Allocating memory in C function insideAreaGridCell.\n");
     PROTECT(results = allocVector(VECSXP, 1));
     UNPROTECT(1);
+    free( shpFileName );
     fclose(newShp);
     remove(TEMP_SHP_FILE);
     return results;
@@ -145,26 +148,30 @@ SEXP insideAreaGridCell(SEXP fileNamePrefix, SEXP dsgnmdIDVec, SEXP cellIDsVec,
     if(combineShpFiles(newShp, dsgnmdID, dsgSize) == -1) {
       PROTECT(results = allocVector(VECSXP, 1));
       UNPROTECT(1);
+      free( dsgnmdID );
+      free( shpFileName );
       fclose(newShp);
       remove(TEMP_SHP_FILE);
       Rprintf("Error: Combining multiple shapefiles in C function insideAreaGridCell.\n");
       return results; 
     }
-    fclose(newShp);
 
   } else {
 
     /* create a temporary .shp file containing the sent .shp file */
     if(createNewTempShpFile(newShp, shpFileName, dsgnmdID, dsgSize) == -1) {
-      PROTECT(results = allocVector(VECSXP, 1));
-      UNPROTECT(1);
+      Rprintf("Error: Creating temporary shapefile in C function insideAreaGridCell.\n");
+      free( dsgnmdID );
+      free( shpFileName );
       fclose(newShp);
       remove(TEMP_SHP_FILE);
-      Rprintf("Error: Creating temporary shapefile in C function insideAreaGridCell.\n");
+      PROTECT(results = allocVector(VECSXP, 1));
+      UNPROTECT(1);
       return results; 
     }
-    fclose(newShp);
   }
+  free( shpFileName );
+  fclose(newShp);
 
   /* initialize the shape struct */
   shape.records = NULL;
@@ -173,6 +180,7 @@ SEXP insideAreaGridCell(SEXP fileNamePrefix, SEXP dsgnmdIDVec, SEXP cellIDsVec,
   /* open the temporary .shp file */
   if((fptr = fopen(TEMP_SHP_FILE, "rb")) == NULL) {
     Rprintf("Error: Opening shape file in C function insideAreaGridCell.\n");
+    remove(TEMP_SHP_FILE);
     PROTECT(results = allocVector(VECSXP, 1));
     UNPROTECT(1);
     return results;
@@ -734,7 +742,7 @@ SEXP insideAreaGridCell(SEXP fileNamePrefix, SEXP dsgnmdIDVec, SEXP cellIDsVec,
             Rprintf("Error: Allocating memory in C function insideAreaGridCell.\n");
             PROTECT(results = allocVector(VECSXP, 1));
             UNPROTECT(1);
-            fclose(newShp);
+            fclose(fptr);
             remove(TEMP_SHP_FILE);
             return results;
           }
@@ -742,7 +750,7 @@ SEXP insideAreaGridCell(SEXP fileNamePrefix, SEXP dsgnmdIDVec, SEXP cellIDsVec,
             Rprintf("Error: Allocating memory in C function insideAreaGridCell.\n");
             PROTECT(results = allocVector(VECSXP, 1));
             UNPROTECT(1);
-            fclose(newShp);
+            fclose(fptr);
             remove(TEMP_SHP_FILE);
             return results;
           }
@@ -759,7 +767,7 @@ SEXP insideAreaGridCell(SEXP fileNamePrefix, SEXP dsgnmdIDVec, SEXP cellIDsVec,
             Rprintf("Error: Allocating memory in C function insideAreaGridCell.\n");
             PROTECT(results = allocVector(VECSXP, 1));
             UNPROTECT(1);
-            fclose(newShp);
+            fclose(fptr);
             remove(TEMP_SHP_FILE);
             return results;
           }
@@ -767,7 +775,7 @@ SEXP insideAreaGridCell(SEXP fileNamePrefix, SEXP dsgnmdIDVec, SEXP cellIDsVec,
             Rprintf("Error: Allocating memory in C function insideAreaGridCell.\n");
             PROTECT(results = allocVector(VECSXP, 1));
             UNPROTECT(1);
-            fclose(newShp);
+            fclose(fptr);
             remove(TEMP_SHP_FILE);
             return results;
           }
@@ -787,10 +795,13 @@ SEXP insideAreaGridCell(SEXP fileNamePrefix, SEXP dsgnmdIDVec, SEXP cellIDsVec,
     } else if(shape.shapeType == POLYGON_Z) {
       free(temp->polyZ->parts);
       free(temp->polyZ->points);
+      free(temp->polyZ->zArray);
+      free(temp->polyZ->mArray);
       free(temp->polyZ);
     } else {
       free(temp->polyM->parts);
       free(temp->polyM->points);
+      free(temp->polyM->mArray);
       free(temp->polyM);
     }
 
@@ -829,6 +840,9 @@ SEXP insideAreaGridCell(SEXP fileNamePrefix, SEXP dsgnmdIDVec, SEXP cellIDsVec,
   if(dsgnmdID) {
     free(dsgnmdID);
   }
+  if(cellIDs) {
+    free(cellIDs);
+  }
   if(xc) {
     free(xc);
   }
@@ -838,6 +852,12 @@ SEXP insideAreaGridCell(SEXP fileNamePrefix, SEXP dsgnmdIDVec, SEXP cellIDsVec,
   if(numIDs) {
     free(numIDs);
   }
+  if(newIDs) {
+    free(newIDs);
+  }
+  if(newAreas) {
+    free(newAreas);
+  }
   fclose(fptr);
   remove(TEMP_SHP_FILE);
   UNPROTECT(5);
